@@ -33,6 +33,10 @@
 #include <libxml/parser.h>
 #include <sys/stat.h>
 
+#ifdef LIBGDA_WIN32
+#include <io.h>
+#endif
+
 #define LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S ".libgda"
 #define LIBGDA_USER_CONFIG_FILE LIBGDA_USER_CONFIG_DIR G_DIR_SEPARATOR_S \
 				"config"
@@ -40,16 +44,16 @@
 /* FIXME: many functions are not thread safe! */
 
 typedef struct {
-    gchar *name;
-    gchar *type;
-    gchar *value;
-    /* gchar *mtime; */
-    /* gchar *muser; */
+	gchar *name;
+	gchar *type;
+	gchar *value;
+	/* gchar *mtime; */
+	/* gchar *muser; */
 } gda_config_entry;
 
 typedef struct {
-    gchar *path;
-    GList *entries;
+	gchar *path;
+	GList *entries;
 } gda_config_section;
 
 typedef struct {
@@ -221,7 +225,7 @@ get_config_client ()
 	 *	  wait until the operation finishes
 	 */
 	if (config_client == NULL){
-		gsize  len;
+		gint len;
 		gchar *full_file;
 		gchar *user_config;
 
@@ -233,7 +237,10 @@ get_config_client ()
 			g_free (full_file);
 
 		}
-
+		if (!g_get_home_dir ()) {
+			/* this can occur on win98, and maybe other win32 platforms */
+			return config_client;
+		}
 		user_config = g_strdup_printf ("%s%s", g_get_home_dir (),
 						LIBGDA_USER_CONFIG_FILE);
 		if (g_file_get_contents (user_config, &full_file, &len, NULL)){
@@ -247,7 +254,11 @@ get_config_client ()
 				dirpath = g_strdup_printf ("%s%s", g_get_home_dir (),
 								LIBGDA_USER_CONFIG_DIR);
 				if (!g_file_test (dirpath, G_FILE_TEST_IS_DIR)){
+#ifdef LIBGDA_WIN32
+					if (mkdir (dirpath))
+#else
 					if (mkdir (dirpath, 0700))
+#endif
 						g_warning ("Error creating directory %s", dirpath);
 				}
 
@@ -761,7 +772,7 @@ gda_config_remove_section (const gchar *path)
 
 	cfg_client = get_config_client ();
 	section = gda_config_search_section (cfg_client->user, path);
-	if (section == NULL){
+	if (section == NULL) {
 		g_warning ("Section %s not found!", path);
 		return;
 	}
@@ -1134,7 +1145,7 @@ gda_config_free_provider_list (GList *list)
 		GdaProviderInfo *provider_info = (GdaProviderInfo *) l->data;
 
 		if (provider_info != NULL)
-			gda_config_free_provider_info (provider_info);
+			gda_provider_info_free (provider_info);
 	}
 
 	g_list_free (list);
@@ -1172,6 +1183,116 @@ gda_config_get_provider_by_name (const gchar *name)
 	return NULL;
 }
 
+/**
+ * gda_config_get_provider_model
+ *
+ * Fills and returns a new #GdaDataModel object using information from all 
+ * providers which are currently installed in the system.
+ *
+ * Rows are separated in 3 columns: 
+ * 'Id', 'Location' and 'Description'. 
+ *
+ * Returns: a new #GdaDataModel object. 
+ */
+GdaDataModel *
+gda_config_get_provider_model (void)
+{
+	GList *prov_list;
+	GList *l;
+	GdaDataModel *model;
+
+	model = gda_data_model_array_new (3);
+	gda_data_model_set_column_title (model, 0, _("Id"));
+	gda_data_model_set_column_title (model, 1, _("Location"));
+	gda_data_model_set_column_title (model, 2, _("Description"));
+
+	/* convert the returned GList into a GdaDataModelArray */
+	prov_list = gda_config_get_provider_list ();
+	for (l = prov_list; l != NULL; l = l->next) {
+		GList *value_list = NULL;
+		GdaProviderInfo *prov_info = (GdaProviderInfo *) l->data;
+
+		g_assert (prov_info != NULL);
+
+		value_list = g_list_append (value_list, gda_value_new_string (prov_info->id));
+		value_list = g_list_append (value_list, gda_value_new_string (prov_info->location));
+		value_list = g_list_append (value_list, gda_value_new_string (prov_info->description));
+
+		gda_data_model_append_row (GDA_DATA_MODEL (model), value_list);
+	}
+	
+	/* free memory */
+	gda_config_free_provider_list (prov_list);
+	
+	return model;
+}
+
+/**
+ * gda_provider_parameter_info_new_full:
+ * @name:
+ * @short_description:
+ * @long_description:
+ * @type:
+ */
+GdaProviderParameterInfo *
+gda_provider_parameter_info_new_full (const gchar *name,
+				      const gchar *short_description,
+				      const gchar *long_description,
+				      GdaValueType type)
+{
+	GdaProviderParameterInfo *param_info;
+
+	param_info = g_new0 (GdaProviderParameterInfo, 1);
+	param_info->name = g_strdup (name);
+	param_info->short_description = g_strdup (short_description);
+	param_info->long_description = g_strdup (long_description);
+	param_info->type = type;
+
+	return param_info;
+}
+
+/**
+ * gda_provider_parameter_info_copy:
+ * @param_info: a #GdaProviderParameterInfo structure.
+ *
+ * Creates a new #GdaProviderParameterInfo structure with the values
+ * contained in the given @param_info argument..
+ *
+ * Returns: a newly allocated #GdaProviderParameterInfo structure.
+ */
+GdaProviderParameterInfo *
+gda_provider_parameter_info_copy (GdaProviderParameterInfo *param_info)
+{
+	GdaProviderParameterInfo *new_info;
+
+	g_return_val_if_fail (param_info != NULL, NULL);
+
+	new_info = g_new0 (GdaProviderParameterInfo, 1);
+	new_info->name = g_strdup (param_info->name);
+	new_info->short_description = g_strdup (param_info->short_description);
+	new_info->long_description = g_strdup (param_info->long_description);
+	new_info->type = param_info->type;
+
+	return new_info;
+}
+
+/**
+ * gda_provider_parameter_info_free:
+ * @param_info: a #GdaProviderParameterInfo structure to be freed.
+ *
+ * Free the memory associated with the given @param_info argument.
+ */
+void
+gda_provider_parameter_info_free (GdaProviderParameterInfo *param_info)
+{
+	g_return_if_fail (param_info != NULL);
+
+	g_free (param_info->name);
+	g_free (param_info->short_description);
+	g_free (param_info->long_description);
+	g_free (param_info);
+}
+
 /**
  * gda_provider_info_copy
  * @src: provider information to get a copy from.
@@ -1198,7 +1319,7 @@ gda_provider_info_copy (GdaProviderInfo *src)
 	/* Deep copy: */
 	list_src = src->gda_params;
 	while (list_src) {
-		list = g_list_append (list, g_strdup (list_src->data));
+		list = g_list_append (list, gda_provider_parameter_info_copy (list_src->data));
 		list_src = g_list_next (list_src);
 	}
 
@@ -1208,20 +1329,20 @@ gda_provider_info_copy (GdaProviderInfo *src)
 }
 
 /**
- * gda_config_free_provider_info
+ * gda_provider_info_free
  * @provider_info: provider information to free.
  *
  * Deallocates all memory associated to the given #GdaProviderInfo.
  */
 void
-gda_config_free_provider_info (GdaProviderInfo *provider_info)
+gda_provider_info_free (GdaProviderInfo *provider_info)
 {
 	g_return_if_fail (provider_info != NULL);
 
 	g_free (provider_info->id);
 	g_free (provider_info->location);
 	g_free (provider_info->description);
-	g_list_foreach (provider_info->gda_params, (GFunc) g_free, NULL);
+	g_list_foreach (provider_info->gda_params, (GFunc) gda_provider_parameter_info_free, NULL);
 	g_list_free (provider_info->gda_params);
 
 	g_free (provider_info);
@@ -1635,6 +1756,19 @@ gda_config_remove_listener (guint id)
 	}
 }
 
+GType
+gda_provider_parameter_info_get_type (void)
+{
+	static GType our_type = 0;
+
+	if (our_type == 0)
+		our_type = g_boxed_type_register_static ("GdaProviderParameterInfo",
+							 (GBoxedCopyFunc) gda_provider_parameter_info_copy,
+							 (GBoxedFreeFunc) gda_provider_parameter_info_free);
+
+	return our_type;
+}
+
 GType
 gda_provider_info_get_type (void)
 {
@@ -1642,10 +1776,10 @@ gda_provider_info_get_type (void)
 
 	if (our_type == 0)
 		our_type = g_boxed_type_register_static ("GdaProviderInfo",
-			(GBoxedCopyFunc) gda_provider_info_copy,
-			(GBoxedFreeFunc) gda_config_free_provider_info);
+							 (GBoxedCopyFunc) gda_provider_info_copy,
+							 (GBoxedFreeFunc) gda_provider_info_free);
 
-  return our_type;
+	return our_type;
 }
 
 GType
@@ -1655,10 +1789,10 @@ gda_data_source_info_get_type (void)
 
 	if (our_type == 0)
 		our_type = g_boxed_type_register_static ("GdaDataSourceInfo",
-			(GBoxedCopyFunc) gda_config_copy_data_source_info,
-			(GBoxedFreeFunc) gda_config_free_data_source_info);
+							 (GBoxedCopyFunc) gda_config_copy_data_source_info,
+							 (GBoxedFreeFunc) gda_config_free_data_source_info);
 
-  return our_type;
+	return our_type;
 }
 
 
