@@ -21,7 +21,6 @@
  */
 
 #if defined(HAVE_CONFIG_H)
-#  include <config.h>
 #endif
 
 #include <libgda/gda-intl.h>
@@ -38,9 +37,9 @@
 
 #define PARENT_TYPE GDA_TYPE_DATA_MODEL
 
-/////////////////////////////////////////////////////////////////////////////
-// Private declarations and functions
-/////////////////////////////////////////////////////////////////////////////
+/*
+ * Private declarations and functions
+ */
 
 static GObjectClass *parent_class = NULL;
 
@@ -59,9 +58,9 @@ static const GdaValue *gda_freetds_recordset_get_value_at (GdaDataModel *model,
                                                            gint         col,
                                                            gint         row);
 
-// Private utility functions
+/* Private utility functions */
 
-// w/o results
+/* w/o results */
 static TDSCOLINFO *gda_freetds_dup_tdscolinfo (TDSCOLINFO *col);
 static GdaRow *gda_freetds_get_current_row(GdaFreeTDSRecordset *recset);
 
@@ -103,7 +102,7 @@ static GdaFieldAttributes
 	                                  gda_freetds_get_value_type (colinfo));
 	gda_field_attributes_set_defined_size (attribs, colinfo->column_size);
 
-	// FIXME:
+	/* FIXME: */
 	gda_field_attributes_set_references (attribs, "");
 	gda_field_attributes_set_primary_key (attribs, FALSE);
 	gda_field_attributes_set_unique_key (attribs, FALSE);
@@ -244,10 +243,12 @@ static GdaRow
 	g_return_val_if_fail (GDA_IS_FREETDS_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, NULL);
 	g_return_val_if_fail (recset->priv->res != NULL, NULL);
-//	g_return_val_if_fail (recset->priv->colcnt 
-//	                      != recset->priv->res->num_cols, NULL);
+/*
+	g_return_val_if_fail (recset->priv->colcnt 
+	                      != recset->priv->res->num_cols, NULL);
+*/
 
-	row = gda_row_new(recset->priv->res->num_cols);
+	row = gda_row_new(GDA_DATA_MODEL (recset), recset->priv->res->num_cols);
 	g_return_val_if_fail (row != NULL, NULL);
 	
 	for (i = 0; i < recset->priv->res->num_cols; i++) {
@@ -276,19 +277,23 @@ static TDSCOLINFO
 	if (copy) {
 		memcpy(copy, col, sizeof(TDSCOLINFO));
 		
-		// set pointers to NULL
+		/* set pointers to NULL */
 		copy->column_nullbind = NULL;
+#ifdef HAVE_FREETDS_VER0_6X
+		copy->column_varaddr = NULL;
+#else
 		copy->varaddr = NULL;
-		copy->column_lenbind = NULL;
 		copy->column_textvalue = NULL;
+#endif
+		copy->column_lenbind = NULL;
 	}
 
 	return copy;
 }
 
-/////////////////////////////////////////////////////////////////////////////
-// Public functions                                                       
-/////////////////////////////////////////////////////////////////////////////
+/*
+ * Public functions                                                       
+ */
 
 GType
 gda_freetds_recordset_get_type (void)
@@ -334,10 +339,28 @@ GdaDataModel
 	recset->priv->tds_cnc = tds_cnc;
 	recset->priv->res = tds_cnc->tds->res_info;
 
+#ifdef HAVE_FREETDS_VER0_63
+       while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds,
+	                                          &tds_cnc->result_type, NULL))
+              == TDS_SUCCEED) {
+               if (tds_cnc->result_type == TDS_ROW_RESULT) {
+                       gint row_type, compute_id;
+
+                       while ((tds_cnc->rc = tds_process_row_tokens(tds_cnc->tds, &row_type, &compute_id))
+#elif defined(HAVE_FREETDS_VER0_6X)
+	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds,
+							 &tds_cnc->result_type))
+	       == TDS_SUCCEED) {
+		if (tds_cnc->result_type == TDS_ROW_RESULT) {
+			gint row_type, compute_id;
+
+			while ((tds_cnc->rc = tds_process_row_tokens(tds_cnc->tds, &row_type, &compute_id))
+#else
 	while ((tds_cnc->rc = tds_process_result_tokens(tds_cnc->tds))
 	       == TDS_SUCCEED) {
 		if (tds_cnc->tds->res_info->rows_exist) {
 			while ((tds_cnc->rc = tds_process_row_tokens(tds_cnc->tds))
+#endif
 			       == TDS_SUCCEED) {
 				recset->priv->res = tds_cnc->tds->res_info;
 				if (columns_set == FALSE) {
